% phi(1:2) 'mq', 'gs', 'iq' the chosen RBF
% ep(1) the shape parameter
% gfunc(1) the type of boundary curve, see gamma1.m and gamma2.m
% tol(1) the tolerance for the integrals in the DtN-conditions.
% kappa(1) the wave number. Complex values not supported(?)
% xl(:,1:2) node points at the left boundary
% xi(:,1:2) node points in the interior 
% xr(:,1:2) node points at the right boundary
% xtb(:,1:2) node points at the top and bottom boundary
% mode(1) If mode = 0, we compute the truncation mu and the DtN-condition,
% otherwise, we use just one mode, and the simplified condition.
% In this case, we should also have gfunc = 0 for a square domain.
%
function [A,fcav] = form_HZ_A(phi,ep,gfunc,tol,kappa,xl,xi,xr,xtb,mode,qr,Psi)
  %tic
  nl = size(xl,1);
  ni = size(xi,1);
  nr = size(xr,1);
  nt = size(xtb,1);
  xall = [xl;xi;xr;xtb];
  N = nl + ni + nr + nt;
  A = zeros(N);
  %
  % Equations for the left boundary starts with negative d/dn
  %
  if (qr)
    xqr = prescaleQR(Psi,xl);
    A(1:nl,:) = -RBF_QR_mat_2D(Psi,'y',xqr);
    A(1:nl,:) = rescale_op(A(1:nl,:),Psi.rr,'y');
  else
    A(1:nl,:) = -dRBF(phi,ep,xl,xall);
  end  
  x2 = 0;
  g1 = gamma1(x2,gfunc);
  g2 = gamma2(x2,gfunc);
  mu_l = floor(kappa*(g2-g1)/pi);
  beta_l = beta(1:mu_l,g1,g2,kappa);
  %other_time=toc;
  %tic
  if (mode>0) % The simplified boundary condition
    %
    % Check compatibility
    %  
    if (gfunc~=0 & gfunc~=4)
      error('The simplified problem needs a rectangular domain')
    elseif (mode<0 | mode>mu_l)
      error('The chosen node number (mode) does not propagate')
    end
    % 
    % The bc is -du/dn - i beta_m u
    % 
    if (qr)
      xqr = prescaleQR(Psi,xl);
      A(1:nl,:) = A(1:nl,:) - i*beta_l(mode)*RBF_QR_mat_2D(Psi,'1',xqr);
    else
      A(1:nl,:) = A(1:nl,:) - i*beta_l(mode)*RBF(phi,ep,xl,xall);
    end
    fcnt_l = 0;
  else  
    for m=1:mu_l
      psi_m = psifun(xl(:,1),m,g1,g2); 
      [ips,fcnt_l(m)]=get_inner_prods(phi,ep,[xl;xi;xr;xtb],m,x2,gfunc, ...
                                          tol,qr,Psi);
      %
      % Outer products, ips = N x 1, psi = nl x 1. psi*ips.' = nl x N. 
      %
      A(1:nl,:) = A(1:nl,:) - i*beta_l(m)*psi_m*ips.';
    end
  end  
  %int_time=toc;
  %tic
  %
  % Equations for the right boundary +d/dn
  %
  if (qr)
    xqr = prescaleQR(Psi,xr);
    A(nl+ni+1:nl+ni+nr,:) = RBF_QR_mat_2D(Psi,'y',xqr);
    A(nl+ni+1:nl+ni+nr,:) = rescale_op(A(nl+ni+1:nl+ni+nr,:),Psi.rr,'y');
  else  
    A(nl+ni+1:nl+ni+nr,:) = dRBF(phi,ep,xr,xall);
  end
  x2 = 1;
  g1 = gamma1(x2,gfunc);
  g2 = gamma2(x2,gfunc);
  mu_r = floor(kappa*(g2-g1)/pi);
  beta_r = beta(1:mu_r,g1,g2,kappa);
  %other_time=other_time+toc;
  %tic
  if (mode>0) % The simplified boundary condition
    % 
    % The bc is du/dn - i beta_m u
    % 
    if (qr)
      xqr = prescaleQR(Psi,xr);
      A(nl+ni+1:nl+ni+nr,:) = A(nl+ni+1:nl+ni+nr,:) ...
          - i*beta_l(mode)*RBF_QR_mat_2D(Psi,'1',xqr);
    else 
      A(nl+ni+1:nl+ni+nr,:) = A(nl+ni+1:nl+ni+nr,:) ...
          - i*beta_l(mode)*RBF(phi,ep,xr,xall);
    end
    fcnt_r = 0;
  else  
    for m=1:mu_r
      psi_m = psifun(xr(:,1),m,g1,g2);
      [ips,fcnt_r(m)]=get_inner_prods(phi,ep,[xl;xi;xr;xtb],m,x2,gfunc,tol,qr,Psi);
      A(nl+ni+1:nl+ni+nr,:) = A(nl+ni+1:nl+ni+nr,:) -i*beta_r(m)*psi_m*ips.';
    end
  end  
  %int_time=int_time+toc;
  %tic
  %
  % Equations for top and bottom
  %
  if (qr)
    xqr = prescaleQR(Psi,xtb);
    A(nl+ni+nr+1:end,:) = RBF_QR_mat_2D(Psi,'1',xqr);
  else
    A(nl+ni+nr+1:end,:) = RBF(phi,ep,xtb,xall);
  end
  %
  % Equations for the inner part
  %
  if (qr)
    xqr = prescaleQR(Psi,xi);
    Atmp = RBF_QR_mat_2D(Psi,'L',xqr);
    Atmp = rescale_op(Atmp,Psi.rr,'L');
    A(nl+1:nl+ni,:) = -Atmp - ...
        kappa^2*RBF_QR_mat_2D(Psi,'1',xqr);
  else
    A(nl+1:nl+ni,:) = -LapRBF(phi,ep,xi,xall) - ...
        kappa^2*RBF(phi,ep,xi,xall);
  end
  disp('The numbers of modes at the left and right boundaries were')
  disp([mu_l,mu_r])

  disp(['The average number of fevals to compute the inner prods were']) 
  disp('Left:')
  disp(fcnt_l)
  disp('Right:')
  disp(fcnt_r)
  fcav = mean([fcnt_l fcnt_r]);
  %other_time=other_time+toc;
  %int_time
  %other_time
  %percentage=int_time./(int_time+other_time)