%
% Using quadl with RBF-QR becomes expensive becasue we compute the whole
% basis every time, when we need just one set of points.
%
% In this case, we can instead use a uniform quadrature, say
% Simpson. Maybe it is always better.
% We can use Richardsson to make an error estimate.
%
function [result,avfcnt]=get_inner_prods(phi,ep,xj,m,x2,gfunc,tol,qr,Psi)
%
% Compute the integral against all RBFs for one of the modes.
%
% phi   = 'mq', 'iq', ...
% ep    = real scalar
% xj    = Nx2 real vector (Could possibly have more dimensions)
% m     = integer (mode number)
% x2    = real scalar. Position of vertical line for integration.
% gfunc = integer (number of gamma-function to use)
% tol   = real scalar (the tolerance in the integration)  
%
  result = zeros(size(xj,1),1);
  g1 = gamma1(x2,gfunc);
  g2 = gamma2(x2,gfunc);
  L = g2-g1;
  alpha = m*pi/L;
  if (qr)
    %
    % First estimate the stepsize in Simpsons formula that leads to an
    % error of size tol using the mth sine function. 
    % E \leq -h^4/180*(b-a)f^(4)(xi)
    % The fourth derivative is alpha^4*sin(). (b-a)=g2-g1 in our
    % problems.
    h = (tol*180/L)^0.25/alpha;
    N = max(3,ceil(L/h)+1);
    if (mod(N,2)==0)
      N = N+1;
    end  
    h = L/(N-1);
    xint = linspace(g1,g2,N)';
    result = intfunc(xint,x2,phi,ep,xj,m,g1,g2,qr,Psi);
    result(2:2:end-1,:) = 2*result(2:2:end-1,:);
    result(3:2:end-2,:) = 4*result(3:2:end-2,:);
    result = h/3*sum(result,1)';
    avfcnt = N;
  else
    for k=1:size(xj,1)
      [result(k),fcnt(k,1)] = quadl(@intfunc,g1,g2,tol,[],x2, ...
                                    phi,ep,xj(k,:),m,g1,g2,qr,Psi);    
    end  
    avfcnt = sum(fcnt)/length(fcnt);
  end  
  %  
% Testcode to see that we get the expected result, Use in the loop as needed.
%
%%     h = (g2-g1)/2000;
%%     x = (g1:h:g2)';
%%     y = intfunc(x,x2,phi,ep,xj(k,:),m,g1,g2);
%%     trapezoid(k) = h*sum(y); % y(0)=y(1)=0

